@@ -120,11 +120,15 @@ struct _EvView {
 
 	EvDocumentModel *model;
 	EvPixbufCache *pixbuf_cache;
+	gsize pixbuf_cache_size;
 	EvPageCache *page_cache;
 	EvHeightToPageCache *height_to_page_cache;
 	EvViewCursor cursor;
 	EvJobRender *current_job;
 
+	GtkRequisition requisition;
+	gboolean       internal_size_request;
+
 	/* Scrolling */
 	GtkAdjustment *hadjustment;
 	GtkAdjustment *vadjustment;
@@ -151,7 +155,8 @@ struct _EvView {
 	gboolean dual_page;
 	gboolean fullscreen;
 	EvSizingMode sizing_mode;
-	cairo_surface_t *loading_text;
+	GtkWidget *loading_window;
+	guint loading_timeout;
 
 	/* Common for button press handling */
 	int pressed_button;
@@ -179,6 +184,9 @@ struct _EvView {
 	/* Annotations */
 	GList             *window_children;
 	EvViewWindowChild *window_child_focus;
+
+	/* Synctex */
+	EvMapping *synctex_result;
 };
 
 struct _EvViewClass {
@@ -194,6 +202,8 @@ struct _EvViewClass {
 	void    (*popup_menu)		  (EvView         *view,
 					   GList          *items);
 	void    (*selection_changed)      (EvView         *view);
+	void    (*sync_source)            (EvView         *view,
+					   EvSourceLink   *link);
 };
 
 void _get_page_size_for_scale_and_rotation (EvDocument *document,
