@@ -79,6 +79,10 @@ void            ev_view_find_changed              (EvView         *view,
 						   gint            page);
 void            ev_view_find_cancel               (EvView         *view);
 
+/* Synctex */
+void            ev_view_highlight_forward_search (EvView       *view,
+						  EvSourceLink *link);
+
 /* Cursor */
 void           ev_view_hide_cursor        (EvView         *view);
 void           ev_view_show_cursor        (EvView         *view);
@@ -95,6 +99,11 @@ gboolean       ev_view_previous_page	  (EvView         *view);
 void	       ev_view_autoscroll_start   (EvView *view);
 void           ev_view_autoscroll_stop    (EvView *view);
 
+gboolean       ev_view_get_page_extents   (EvView       *view,
+                                           gint          page,
+                                           GdkRectangle *page_area,
+                                           GtkBorder    *border);
+
 G_END_DECLS
 
 #endif /* __EV_VIEW_H__ */
