@@ -146,6 +146,28 @@ launch_previewer (void)
 	return retval;
 }
 
+static gchar *
+get_label_from_filename (const gchar *filename)
+{
+	GFile   *file;
+	gchar   *label;
+	gboolean exists;
+
+	label = g_strrstr (filename, "#");
+	if (!label)
+		return NULL;
+
+	/* Filename contains a #, check
+	 * whether it's part of the path
+	 * or a label
+	 */
+	file = g_file_new_for_commandline_arg (filename);
+	exists = g_file_query_exists (file, NULL);
+	g_object_unref (file);
+
+	return exists ? NULL : label;
+}
+
 static void
 load_files (const char **files)
 {
@@ -179,7 +201,7 @@ load_files (const char **files)
 		const gchar *app_uri;
 
 		filename = files[i];
-		label = strchr (filename, '#');
+		label = get_label_from_filename (filename);
 		if (label) {
 			*label = 0;
 			label++;
@@ -216,11 +238,6 @@ main (int argc, char *argv[])
 	GOptionContext *context;
 	GError         *error = NULL;
 
-#ifdef WITH_GCONF
-	/* Remove when porting is done */
-	g_setenv ("GSETTINGS_BACKEND", "gconf", FALSE);
-#endif
-
 #ifdef G_OS_WIN32
 
     if (fileno (stdout) != -1 &&
@@ -252,7 +269,7 @@ main (int argc, char *argv[])
 
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
-	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bindtextdomain (GETTEXT_PACKAGE, ev_get_locale_dir());
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 #endif
@@ -299,14 +316,13 @@ main (int argc, char *argv[])
 
 	ev_application_load_session (EV_APP);
 	load_files (file_arguments);
-	if (ev_application_has_window (EV_APP)) {
-		/* Change directory so we don't prevent unmounting in case the initial cwd
-		 * is on an external device (see bug #575436)
-		 */
-		g_chdir (g_get_home_dir ());
 
-		gtk_main ();
-	}
+	/* Change directory so we don't prevent unmounting in case the initial cwd
+	 * is on an external device (see bug #575436)
+	 */
+	g_chdir (g_get_home_dir ());
+
+	gtk_main ();
 
 	ev_shutdown ();
 	ev_stock_icons_shutdown ();
